@@ -9,7 +9,7 @@ use Config::JFDI;
 use Carp qw/croak/;
 use Data::Dumper;
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 subtype MyConf => as 'HashRef';
 coerce 'MyConf'
@@ -88,6 +88,12 @@ has _plugin_ns => (
                 default  => sub{ 'Backend' },
                   );
 
+has _type_field => (
+                is       => 'rw',
+                isa      => 'Str',
+                default  => sub{ 'type' },
+                  );
+
 has _search_field => (
                 is       => 'rw',
                 isa      => 'Str',
@@ -117,6 +123,7 @@ has root => (
 );
 
 
+
 sub _load_backend {
   my $self = shift;
   my $backend = $self->config->{'backend'}->{name};
@@ -168,10 +175,17 @@ sub _build_tree {
       next;
     }
 
+    my $parent_type = $parent->{$self->_type_field} ||'';
+    my $node_type   = $node->{$self->_type_field}   ||'';
 
-    my $children_parent = defined $parent->{children} ? $parent->{children} : [];
+    my $key_children;
+    if ( "$node_type" eq "$parent_type") {
+            $key_children = 'children' }
+    else {  $key_children = 'children_' . $node_type }
+
+    my $children_parent = defined $parent->{$key_children} ? $parent->{$key_children} : [];
     push(@$children_parent, $node );
-    $parent->{children} = $children_parent;
+    $parent->{$key_children} = $children_parent;
     $node->{parent}     = $parent;
   }
 }
@@ -281,7 +295,7 @@ sub search {
   my $tree = $self->tree;
   foreach my $id  ( sort {$a <=> $b} keys %$tree ) {
 
-    my $found=1;
+    my $found = 1;
     foreach my $key ( keys %$args ) {
       my $current;
       if ( $key =~ m/(.*)\.(.*)/) {
@@ -292,7 +306,12 @@ sub search {
         else { next }
       }
       else {
-        $current = $tree->{$id}->{$key};
+          if ( defined $tree->{$id}->{$key} ){
+              $current = $tree->{$id}->{$key};
+          }
+          else {
+              die "'$key' is not a key hash [node id:$id]";
+          }
       }
       my $value = $args->{$key};
       if ( $current ne $value ) {
@@ -388,7 +407,7 @@ sub traverse {
 
   my $nofunc = 0;
   if ( ! $funcref ) {
-    $nofunc=1;
+    $nofunc = 1;
     $funcref = sub {    my ($node, $args) = @_;
                         $args->{_each_nodes} = []
                           if ( ! defined $args->{_each_nodes});
@@ -449,7 +468,7 @@ sub add {
   my $update_children = 0;
   if ( $parent ) {
       if ( $position ) {
-          splice @{$parent->{children}}, $position-1, 0, $node;
+          splice @{$parent->{children}}, $position -1, 0, $node;
           $update_children = 1;
       }
       else {
@@ -500,10 +519,10 @@ sub update {
 
       # if update 'position'
       if ( $k eq $self->_position_field ) {
-          # delete child in parent in previous positioh
-          my $old = splice(@$children, $previous-1, 1);
+          # delete child in parent in previous position
+          my $old = splice(@$children, $previous -1, 1);
           # add child in parent in new positioh
-          splice(@$children,$datas->{$k}-1,0, $old);
+          splice(@$children,$datas->{$k} -1,0, $old);
           $self->_update_children_position(@{$node->{parent}->{children}})
               if ( defined $node->{$self->_position_field});
       }
@@ -563,7 +582,7 @@ TreePath - Simple Tree Path!
 
 =head2 new($method => $value)
 
- # for now there are two backend : DBIX and File
+ # for now there are two backends : DBIX and File
  $tp = TreePath->new( conf => 't/conf/treefromdbix.yml')
 
  # see t/conf/treepath.yml for hash structure
